//
//  WTInstantTrackerViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 17/11/16.
//  Copyright © 2016 Wikitude. All rights reserved.
//

#import "WTInstantTrackerViewController.h"

#import <WikitudeNativeSDK/WikitudeNativeSDK.h>

#import "WikitudeLicense.h"
#import "StrokedRectangle.h"
#import "ExternalRenderer.h"


@interface WTInstantTrackerViewController () <WTWikitudeNativeSDKDelegate, WTInstantTrackerDelegate>

@property (nonatomic, strong) WTWikitudeNativeSDK                   *wikitudeSDK;
@property (nonatomic, strong) WTInstantTracker                      *instantTracker;

@property (nonatomic, strong) EAGLContext                           *sharedWikitudeEAGLCameraContext;

@property (nonatomic, copy) WTWikitudeUpdateHandler                 wikitudeUpdateHandler;
@property (nonatomic, copy) WTWikitudeDrawHandler                   wikitudeDrawHandler;

@property (nonatomic, assign) BOOL                                  isTracking;
@property (nonatomic, assign) WTInstantTrackingState                trackingState;

@property (nonatomic, strong) ExternalRenderer                      *renderer;
@property (nonatomic, strong) StrokedRectangle                      *renderableRectangle;

@end

@implementation WTInstantTrackerViewController

#pragma mark UIViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.renderer = [[ExternalRenderer alloc] init];
    self.renderableRectangle = [[StrokedRectangle alloc] init];
    self.renderableRectangle.scale = 0.45;

    self.wikitudeSDK = [[WTWikitudeNativeSDK alloc] initWithRenderingMode:WTRenderingMode_External delegate:self];
    [self.wikitudeSDK setLicenseKey:kWTLicenseKey];

    self.deviceHeightAboveGroundSlider.transform = CGAffineTransformMakeRotation(M_PI_2);

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationWillResignActiveNotification:) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationDidBecomeActiveNotification:) name:UIApplicationDidBecomeActiveNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    [self.renderer setupRenderingWithLayer:[self.eaglView eaglLayer]];
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            self.instantTracker = [self.wikitudeSDK.trackerManager createInstantTracker:self configuration:nil];
            self.trackingState = WTInstantTrackerInitializing;
        }
    }];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    [self.wikitudeSDK stop];
    [self.renderableRectangle releaseProgram];
    [self.renderer stopRenderLoop];
    [self.renderer teardownRendering];

    _sharedWikitudeEAGLCameraContext = nil;
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];

    [self.wikitudeSDK shouldTransitionToSize:size withTransitionCoordinator:coordinator];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Actions
- (IBAction)toggleInstantTrackingState:(id)sender
{
    NSLog(@"toggle instant tracking state");
    if ( [[self.instantTrackingButton currentTitle] isEqualToString:@"Start Tracking"] )
    {
        if ( WTInstantTrackerInitializing == self.trackingState )
        {
            [self.instantTrackingButton setTitle:@"Start Initialization" forState:UIControlStateNormal];
            [self.instantTracker setActiveInstantTrackingState:WTInstantTrackerTracking];
        }
        else
        {
            NSLog(@"Cannot change to tracking state because instant tracker didn't change to initialization state yet.");
        }
    }
    else
    {
        if ( WTInstantTrackerTracking == self.trackingState )
        {
            [self.instantTrackingButton setTitle:@"Start Tracking" forState:UIControlStateNormal];
            [self.instantTracker setActiveInstantTrackingState:WTInstantTrackerInitializing];
        }
        else
        {
            NSLog(@"Cannot change to initialization state because instant tracker didn't change to tracking state yet.");
        }
    }
}

- (IBAction)updateDeviceHeightAboveGround:(UISlider *)sender
{
    [self.instantTracker setDeviceHeightAboveGround:@(sender.value)];
}

#pragma mark - ExternalRenderer render loop
- (ExternalRenderBlock)renderBlock
{
    return ^ (CADisplayLink *displayLink) {
        if ( self.wikitudeUpdateHandler
            &&
            self.wikitudeDrawHandler )
        {
            self.wikitudeUpdateHandler();
            self.wikitudeDrawHandler();
        }

        [self.renderer bindBuffer];

        if ( _isTracking )
        {
            [self.renderableRectangle drawInContext:[self.renderer internalContext]];
        }
    };
}

#pragma mark - Notifications
- (void)didReceiveApplicationWillResignActiveNotification:(NSNotification *)notification
{
    [self.wikitudeSDK stop];

    [self.renderer stopRenderLoop];
}

- (void)didReceiveApplicationDidBecomeActiveNotification:(NSNotification *)notification
{
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
    }];
}

#pragma mark - Delegation
#pragma mark WTWikitudeNativeSDKDelegte
- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalUpdateHandler:(WTWikitudeUpdateHandler __nonnull)updateHandler
{
    self.wikitudeUpdateHandler = updateHandler;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalDrawHandler:(WTWikitudeDrawHandler __nonnull)drawHandler
{
    self.wikitudeDrawHandler = drawHandler;
}

- (EAGLContext *)eaglContextForVideoCameraInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    if (!_sharedWikitudeEAGLCameraContext )
    {
        EAGLContext *rendererContext = [self.renderer internalContext];
        self.sharedWikitudeEAGLCameraContext = [[EAGLContext alloc] initWithAPI:[rendererContext API] sharegroup:[rendererContext sharegroup]];
    }
    return self.sharedWikitudeEAGLCameraContext;
}

- (CGRect)eaglViewSizeForExternalRenderingInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    return self.eaglView.bounds;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didEncounterInternalError:(NSError * __nonnull)error
{
    NSLog(@"Internal Wikitude SDK error encounterd. %@", [error localizedDescription]);
}

#pragma mark WTInstantTrackerDelegate
- (void)instantTracker:(nonnull WTInstantTracker *)instantTracker didChangeState:(WTInstantTrackingState)newState
{
    NSLog(@"didChangeState %ld", (long)newState);
    _trackingState = newState;
    dispatch_async(dispatch_get_main_queue(), ^{
        if ( WTInstantTrackerTracking == newState )
        {
            self.deviceHeightAboveGroundSlider.alpha = 0.0;
        }
        else
        {
            self.deviceHeightAboveGroundSlider.alpha = 1.0;
        }
    });
    
    if (_trackingState == WTInstantTrackerInitializing) {
        [self.renderableRectangle setColor:[UIColor colorWithRed:1.00f green:0.58f blue:0.16f alpha:1.0f]];
    } else {
        [self.renderableRectangle setColor:[UIColor colorWithRed:0.41f green:0.60f blue:0.76f alpha:1.0f]];
    }
}

- (void)instantTracker:(nonnull WTInstantTracker *)instantTracker didChangeInitializationPose:(nonnull WTInitializationPose *)pose
{
    _isTracking = YES;
    
    [self.renderableRectangle setProjectionMatrix:pose.projection];
    [self.renderableRectangle setModelViewMatrix:pose.modelView];
}

- (void)instantTrackerDidStartTracking:(nonnull WTInstantTracker *)instantTracker
{
    NSLog(@"didStartTracking");
    _isTracking = YES;
}

- (void)instantTracker:(nonnull WTInstantTracker *)instantTracker didTrack:(nonnull WTInstantTarget *)target
{
    [self.renderableRectangle setProjectionMatrix:target.projection];
    [self.renderableRectangle setModelViewMatrix:target.modelView];
}

- (void)instantTrackerDidStopTracking:(nonnull WTInstantTracker *)instantTracker
{
    NSLog(@"didStopTracking");
    _isTracking = NO;
}

@end
